﻿using AirFighter.Drawnings;
using AirFighter.MovementStrategy;
using AirFighter.DrawningObjects;
using AirFighter.Generics;
using AirFighter.MovementStrategy;

namespace AirFighter.Generics
{
    internal class AirFighterGenericCollection<T, U>
    where T : DrawningAirFighter
    where U : IMoveableObject
    {
        public IEnumerable<T?> GetAirFighter => _collection.GetAirFighter();

        private readonly int _pictureWidth;

        private readonly int _pictureHeight;

        private readonly int _placeSizeWidth = 210;

        private readonly int _placeSizeHeight = 180;

        private readonly SetGeneric<T> _collection;
        public AirFighterGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }
        public static int? operator +(AirFighterGenericCollection<T, U> collect, T? obj)
        {
            if (obj == null)
            {
                return -1;
            }
            return collect?._collection.Insert(obj);
        }
        public static T operator -(AirFighterGenericCollection<T, U> collect, int pos)
        {
            T? obj = collect._collection[pos];
            if (obj != null)
            {
                collect?._collection.Remove(pos);
            }
            return obj;
        }
        public U? GetU(int pos)
        {
            return (U?)_collection[pos]?.GetMoveableObject;
        }
        public Bitmap ShowAirFighter()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 2);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }
        private void DrawObjects(Graphics g)
        {
            int width = _pictureWidth / _placeSizeWidth;
            int height = _pictureHeight / _placeSizeHeight;
            for (int i = 0; i < _collection.Count; i++)
                foreach (var airfighter in _collection.GetAirFighter())
                {
                    if (airfighter != null)
                    {
                        T? fighter = _collection[i];
                        if (fighter == null)
                            continue;
                        int row = height - 1 - (i / width);
                        int col = width - 1 - (i % width);
                        fighter.SetPosition(col * _placeSizeWidth + 10, row * _placeSizeHeight + 5);
                        fighter.DrawTransport(g);
                    }
                }
        }
    }
}

