﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using AirFighter.DrawningObjects;
using AirFighter.MovementStrategy;
using AirFighter.Generics;
using System.IO;
using AirFighter.Exceptions;


namespace AirFighter.Generics
{
    internal class AirFighterGenericStorage
    {
        readonly Dictionary<string, AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>> _fighterStorages;

        public List<string> Keys => _fighterStorages.Keys.ToList();

        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        public AirFighterGenericStorage(int pictureWidth, int pictureHeight)
        {
            _fighterStorages = new Dictionary<string, AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }
        public void AddSet(string name)
        {
            if (_fighterStorages.ContainsKey(name)) return;
            _fighterStorages[name] = new AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>(_pictureWidth, _pictureHeight);
        }

        public void DelSet(string name)
        {
            if (!_fighterStorages.ContainsKey(name)) return;
            _fighterStorages.Remove(name);
        }

        public AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>?
        this[string ind]
        {
            get
            {
                if (_fighterStorages.ContainsKey(ind)) return _fighterStorages[ind];
                return null;
            }
        }

        private static readonly char _separatorForKeyValue = '|';

        private readonly char _separatorRecords = ';';

        private static readonly char _separatorForObject = ':';
        public void SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            StringBuilder data = new();
            foreach (KeyValuePair<string, AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter>> record in _fighterStorages)
            {
                StringBuilder records = new();
                foreach (DrawningAirFighter? elem in record.Value.GetAirFighter)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                throw new Exception("Невалидная операция, нет данных для сохранения");
            }

            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.Write($"fighterStorage{Environment.NewLine}{data}");
            }
        }
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exception("Файл не найден");
            }
            using (StreamReader reader = new StreamReader(filename))
            {
                string cheker = reader.ReadLine();
                if (cheker == null)
                {
                    throw new Exception("Нет данных для загрузки");
                }
                if (!cheker.StartsWith("fighterStorage"))
                {
                    throw new Exception("Неверный формат ввода");
                }
                _fighterStorages.Clear();
                string strs;
                bool firstinit = true;
                while ((strs = reader.ReadLine()) != null)
                {
                    if (strs == null && firstinit)
                    {
                        throw new Exception("Нет данных для загрузки");
                    }
                    if (strs == null)
                    {
                        break;
                    }
                    firstinit = false;
                    string name = strs.Split(_separatorForKeyValue)[0];
                    AirFighterGenericCollection<DrawningAirFighter, DrawningObjectAirFighter> collection = new(_pictureWidth, _pictureHeight);
                    foreach (string data in strs.Split(_separatorForKeyValue)[1].Split(_separatorRecords))
                    {
                        DrawningAirFighter? fighter =
                        data?.CreateDrawningAirFighter(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (fighter != null)
                        {
                            try { _ = collection + fighter; }
                            catch (AirFighterNotFoundException e)
                            {
                                throw e;
                            }
                            catch (StorageOverflowException e)
                            {
                                throw e;
                            }
                        }
                    }
                    _fighterStorages.Add(name, collection);
                }
            }
        }
    }
}
