﻿using System;
using System.Collections.Generic;
using System.Drawing.Drawing2D;
using System.IO;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using AirFighter.Entities;
using AirFighter.Drawnings;
using System.Drawing.Drawing2D;
using AirFighter.MovementStrategy;

namespace AirFighter.DrawningObjects
{
    public class DrawningAirFighter
    {
        public EntityAirFighter? EntityAirFighter { get; protected set; }

        private int _pictureWidth;

        private int _pictureHeight;

        protected int _startPosX;

        protected int _startPosY;

        protected readonly int _fighterWidth = 195;

        protected readonly int _fighterHeight = 166;
        
        public DrawningAirFighter(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _fighterWidth || height < _fighterHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityAirFighter = new EntityAirFighter(speed, weight, bodyColor);
        }

        protected DrawningAirFighter(int speed, double weight, Color bodyColor, int
        width, int height, int fighterWidth, int fighterHeight)
        {
            if (width <= _pictureWidth || height <= _pictureHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _fighterWidth = fighterWidth;
            _fighterHeight = fighterHeight;
            EntityAirFighter = new EntityAirFighter(speed, weight, bodyColor);
        }
        public void SetPosition(int x, int y)
        {
            if ((x > 0) && (x < _pictureWidth))
                _startPosX = x;
            else _startPosX = 0;
            if ((y > 0) && (y < _pictureHeight))
                _startPosY = y;
            else _startPosY = 0;
            _startPosX = x;
            _startPosY = y;
        }
        public IMoveableObject GetMoveableObject => new DrawningObjectAirFighter(this);
        public int GetPosX => _startPosX;
        public int GetPosY => _startPosY;
        public int GetWidth => _fighterWidth;
        public int GetHeight => _fighterHeight;
        public bool CanMove(DirectionAirFighter direction)
        {
            if (EntityAirFighter == null)
            {
                return false;
            }
            return direction switch
            {
                DirectionAirFighter.Left => _startPosX - EntityAirFighter.Step > 0,
               
                DirectionAirFighter.Up => _startPosY - EntityAirFighter.Step > 7,
              
                DirectionAirFighter.Right => _startPosX + EntityAirFighter.Step + _fighterWidth < _pictureWidth,// TODO: Продумать логику
             
                DirectionAirFighter.Down => _startPosY + EntityAirFighter.Step + _fighterHeight < _pictureHeight,// TODO: Продумать логику
                _ => false,
            };
        }
        public void MoveTransport(DirectionAirFighter direction)
        {
            if (!CanMove(direction) || EntityAirFighter == null)
            {
                return;
            }
            switch (direction)
            {
                case DirectionAirFighter.Left:
                    _startPosX -= (int)EntityAirFighter.Step;
                    break;
                case DirectionAirFighter.Up:
                    _startPosY -= (int)EntityAirFighter.Step;
                    break;
                case DirectionAirFighter.Right:
                    _startPosX += (int)EntityAirFighter.Step;
                    break;
                case DirectionAirFighter.Down:
                    _startPosY += (int)EntityAirFighter.Step;
                    break;
            }
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityAirFighter == null)
            {
                return;
            }
            Pen pen = new(EntityAirFighter.BodyColor, 2);
            Brush brushBlack = new SolidBrush(EntityAirFighter.BodyColor);

            PointF[] front = {
                new(_startPosX + 160, _startPosY + 69),
                new(_startPosX + 195, _startPosY + 83),
                new(_startPosX + 160, _startPosY + 97)
            };

            PointF[] tailTop = {
                new(_startPosX, _startPosY + 30),
                new(_startPosX, _startPosY + 70),
                new(_startPosX + 25, _startPosY + 70),
                new(_startPosX + 25, _startPosY + 55)
            };

            PointF[] tailBottom = {
                new(_startPosX, _startPosY + 96),
                new(_startPosX, _startPosY + 136),
                new(_startPosX + 25, _startPosY + 111),
                new(_startPosX + 25, _startPosY + 96)
            };

            PointF[] wingTop =
            {
                new(_startPosX + 100, _startPosY),
                new(_startPosX + 100, _startPosY + 70),
                new(_startPosX + 75, _startPosY + 70),
                new(_startPosX + 90, _startPosY),
            };
            PointF[] wingBottom =
            {
                new(_startPosX + 100, _startPosY + 96),
                new(_startPosX + 100, _startPosY + 166),
                new(_startPosX + 90, _startPosY + 166),
                new(_startPosX + 75, _startPosY + 96),
            };

            g.DrawPolygon(pen, tailTop);
            g.DrawPolygon(pen, tailBottom);
            g.DrawPolygon(pen, wingTop);
            g.DrawPolygon(pen, wingBottom);
            g.DrawRectangle(pen, _startPosX, _startPosY + 70, 160, 26);
            g.FillPolygon(brushBlack, front);
        }
    }  
}
       

  
