﻿
using AirFighter.DrawningObjects;
using AirFighter.Drawnings;
using AirFighter.Generics;
using AirFighter.MovementStrategy;
using Microsoft.Extensions.Logging;



namespace AirFighter
{
    /// <summary>
    /// Форма для работы с набором объектов класса DrawningAirFighter
    /// </summary>
    public partial class FormAirFighterCollection : Form
    {
        /// <summary>
        /// Набор объектов
        /// </summary>
        private readonly AirFighterGenericStorage _storage;
        private readonly ILogger _logger;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormAirFighterCollection(ILogger<FormAirFighterCollection> logger)
        {
            InitializeComponent();
            _storage = new AirFighterGenericStorage(pictureBoxCollection.Width,
            pictureBoxCollection.Height);
            _logger = logger;
        }
        /// <summary>
        /// Заполнение listBoxObjects
        /// </summary>
        private void ReloadObjects()
        {
            int index = listBoxStorage.SelectedIndex;
            listBoxStorage.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorage.Items.Add(_storage.Keys[i]);
            }
            if (listBoxStorage.Items.Count > 0 && (index == -1 || index
            >= listBoxStorage.Items.Count))
            {
                listBoxStorage.SelectedIndex = 0;
            }
            else if (listBoxStorage.Items.Count > 0 && index > -1 &&
            index < listBoxStorage.Items.Count)
            {
                listBoxStorage.SelectedIndex = index;
            }
        }
        /// <summary>
        /// Добавление набора в коллекцию
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxStorageName.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogWarning("Пустое название набора");
                return;
            }
            _storage.AddSet(textBoxStorageName.Text);
            ReloadObjects();
            _logger.LogInformation($"Добавлен набор:{textBoxStorageName.Text}");
        }
        /// <summary>
        /// Выбор набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listBoxStorage_SelectedIndexChanged(object sender,
        EventArgs e)
        {
            pictureBoxCollection.Image =
            _storage[listBoxStorage.SelectedItem?.ToString() ?? string.Empty]?.ShowAirFighter();
        }
        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorage.SelectedIndex == -1)
            {
                _logger.LogWarning("Удаление невыбранного набора");
                return;
            }
            string name = listBoxStorage.SelectedItem.ToString() ?? string.Empty;
            if (MessageBox.Show($"Удалить объект {listBoxStorage.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)

            {
                _storage.DelSet(listBoxStorage.SelectedItem.ToString()
                ?? string.Empty);
                ReloadObjects();
                _logger.LogInformation($"Удален набор: {name}");
            }
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
       
        private void ButtonAddAirFighter_Click(object sender, EventArgs e)
        {
            if (listBoxStorage.SelectedIndex == -1)
            {
                return;
            }
            var formAirFighterConfig = new FormAirFighterConfig();
            formAirFighterConfig.AddEvent(fighter =>
            {
                var obj = _storage[listBoxStorage.SelectedItem.ToString() ?? string.Empty];
                if (obj == null)
                {
                    _logger.LogWarning("Добавление пустого объекта");
                    return;
                }
                try
                {
                    _ = obj + fighter;

                    MessageBox.Show("Объект добавлен");
                    pictureBoxCollection.Image = obj.ShowAirFighter();
                    _logger.LogInformation($"Добавлен объект в набор {listBoxStorage.SelectedItem.ToString()}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Не удалось добавить объект");
                    _logger.LogWarning($"{ex.Message} в наборе {listBoxStorage.SelectedItem.ToString()}");
                }
            });
            formAirFighterConfig.Show();
        }


        /// <summary>
        /// Удаление объекта из набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveAirFighter_Click(object sender, EventArgs e)
        {
            if (listBoxStorage.SelectedIndex == -1)
            {
                _logger.LogWarning("Удаление объекта из несуществующего набора");
                return;
            }
            var obj = _storage[listBoxStorage.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxNumber.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCollection.Image = obj.ShowAirFighter();
                _logger.LogInformation($"Удален объект из набора {listBoxStorage.SelectedItem.ToString()}");
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");                                              
                _logger.LogWarning($"Не удалось удалить объект из набора {listBoxStorage.SelectedItem.ToString()}");
            }
        }
        /// <summary>
        /// Обновление рисунка по набору
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefreshCollection_Click(object sender, EventArgs
        e)
        {
            if (listBoxStorage.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorage.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCollection.Image = obj.ShowAirFighter();
        }

        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.SaveData(saveFileDialog1.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Сохранение наборов в файл {saveFileDialog1.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не сохранилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не удалось сохранить наборы с ошибкой: {ex.Message}");
                }
            }
        }

        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storage.LoadData(openFileDialog1.FileName);
                    ReloadObjects();
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation($"Загрузились наборы из файла {openFileDialog1.FileName}");
                }
                catch (Exception ex)
                {
                    MessageBox.Show($"Не загрузилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogWarning($"Не удалось загрузить наборы с ошибкой: {ex.Message}");
                }
            }
        }
    }
}

