﻿
using AirFighter.Drawnings;
using AirFighter.Entities;
using AirFighter;
using AirFighter.DrawningObjects;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;

namespace AirFighter
{
    /// <summary>
    /// Форма создания объекта
    /// </summary>
    public partial class FormAirFighterConfig : Form
    {
        /// <summary>
        /// Переменная-выбранная установка
        /// </summary>
        DrawningAirFighter? _fighter = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event Action<DrawningAirFighter> EventAddAirFighter;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormAirFighterConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += panelColor_MouseDown;
            panelPurple.MouseDown += panelColor_MouseDown;
            panelGray.MouseDown += panelColor_MouseDown;
            panelGreen.MouseDown += panelColor_MouseDown;
            panelRed.MouseDown += panelColor_MouseDown;
            panelWhite.MouseDown += panelColor_MouseDown;
            panelYellow.MouseDown += panelColor_MouseDown;
            panelBlue.MouseDown += panelColor_MouseDown;
            // TODO buttonCancel.Click with lambda
            buttonCancel.Click += (sender, e) => Close();
        }
        /// <summary>
        /// Отрисовать установку
        /// </summary>
        private void DrawAirFighter()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _fighter?.SetPosition(5, 5);
            _fighter?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawningAirFighter> ev)
        {
            if (EventAddAirFighter == null)
            {
                EventAddAirFighter = new Action<DrawningAirFighter>(ev);
            }
            else
            {
                EventAddAirFighter += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _fighter = new DrawningAirFighter((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width,
                   pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _fighter = new DrawningAirFighterMilitary((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxWing.Checked,
                   checkBoxRocket.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawAirFighter();
        }

        /// <summary>
        /// Отправляем цвет с панели
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void panelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        // TODO Реализовать логику смены цветов: основного и дополнительного (для продвинутого объекта)
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_fighter != null)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    _fighter.EntityAirFighter.BodyColor = (Color)e.Data.GetData(typeof(Color));

                }
                DrawAirFighter();
            }
        }

        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (_fighter != null && _fighter.EntityAirFighter is EntityAirFighterMilitary entityustabat)
            {
                labelDopColor.AllowDrop = true;
            }
            else
                labelDopColor.AllowDrop = false;

            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_fighter != null && _fighter.EntityAirFighter is EntityAirFighterMilitary entityAirFighterMilitary)
            {
                if (e.Data.GetDataPresent(typeof(Color)))
                {
                    entityAirFighterMilitary.AdditionalColor = (Color)e.Data.GetData(typeof(Color));

                }
                DrawAirFighter();
            }
        }
        /// <summary>
        /// Добавление установки
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddAirFighter?.Invoke(_fighter);
            Close();
        }
    }

}