﻿using System.Numerics;
using AirFighter.Exceptions;

namespace AirFighter.Generics
{
    internal class SetGeneric<T>
    where T : class
    {
        private readonly List<T?> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        public int Insert(T fighter)
        {
            if (_places.Count == 0)
            {
                _places.Add(fighter);
                return 0;
            }
            else
            {
                if (_places.Count < _maxCount)
                {
                    _places.Add(fighter);
                    for (int i = 0; i < _places.Count; i++)
                    {
                        T temp = _places[i];
                        _places[i] = _places[_places.Count - 1];
                        _places[_places.Count - 1] = temp;
                    }
                    return 0;
                }
                else
                {
                    throw new StorageOverflowException(_places.Count);
                }
            }

        }
        public bool Insert(T fighter, int position)
        {
            if (position < 0 || position >= _maxCount)
                throw new AirFighterNotFoundException(position);

            if (Count >= _maxCount)
                throw new StorageOverflowException(position);
            _places.Insert(0, fighter);
            return true;
        }
        public bool Remove(int position)
        {
            if (position < 0 || position > _maxCount || position >= Count)
                throw new AirFighterNotFoundException();
            if (_places[position] == null)
            {
                throw new AirFighterNotFoundException();
            }
            _places[position] = null;
            return true;
        }
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                _places[position] = value;
            }
        }
        public IEnumerable<T?> GetAirFighter(int? maxAirFighter = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxAirFighter.HasValue && i == maxAirFighter.Value)
                {
                    yield break;
                }
            }
        }
    }
}

